<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Model\Config\Seo;

use Magento\Framework\DataObjectFactory;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use MageWorx\SeoBase\Helper\Data;
use MageWorx\SeoBase\Helper\Hreflangs;

use function is_string;

/**
 *
 */
class MageWorxSeoHrefLangConfigurationSource implements HrefLangConfigurationSourceInterface
{
    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var \Magento\Framework\Module\ModuleListInterface
     */
    private $moduleList;

    /**
     * @var \Magento\Framework\DataObject
     */
    private $parameters;

    /**
     * @var \Magento\Framework\DataObjectFactory
     */
    private $dataObjectFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \FiloBlu\Refilo\Model\Config\Seo\HrefLangConfigurationSourceInterface
     */
    private $defaultConfiguration;

    /**
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \FiloBlu\Refilo\Model\Config\Seo\HrefLangConfigurationSourceInterface $defaultConfiguration
     */
    public function __construct(
        ModuleListInterface $moduleList,
        ObjectManagerInterface $objectManager,
        DataObjectFactory $dataObjectFactory,
        StoreManagerInterface $storeManager,
        HrefLangConfigurationSourceInterface $defaultConfiguration
    ) {
        $this->objectManager = $objectManager;
        $this->moduleList = $moduleList;
        $this->dataObjectFactory = $dataObjectFactory;
        $this->storeManager = $storeManager;
        $this->defaultConfiguration = $defaultConfiguration;
    }

    /**
     * @param $key
     * @param $store
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getValue($key, $store)
    {
        if (is_numeric($store) || $store === null) {
            $index = (int)$store;
            return $this->getConfiguration()->getData("by_id_{$index}_$key");
        }

        if ($store instanceof StoreInterface) {
            $code = $store->getCode();
            return $this->getConfiguration()->getData("by_code_{$code}_$key");
        }

        if (is_string($store)) {
            return $this->getConfiguration()->getData("by_code_{$store}_$key");
        }

        return null;
    }

    /**
     * @return \Magento\Framework\DataObject
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getConfiguration()
    {
        if ($this->parameters !== null) {
            return $this->parameters;
        }

        $this->parameters = $this->dataObjectFactory->create();

        $hrefLangHelper = null;
        $helperData = null;
        if ($this->moduleList->has('MageWorx_SeoBase')) {
            /** @var \MageWorx\SeoBase\Helper\Hreflangs $hrefLangHelper */
            $hrefLangHelper = $this->objectManager->get(Hreflangs::class);
            /** @var \MageWorx\SeoBase\Helper\Data $helperData */
            $helperData = $this->objectManager->get(Data::class);
        }


        // Prepare default values

        foreach ($this->storeManager->getStores(true) as $store) {
            $storeId = $store->getId();
            $code = $store->getCode();
            // By code
            $this->parameters->setData("by_code_{$code}_enabled", false);
            $this->parameters->setData("by_code_{$code}_enabled_category", false);
            $this->parameters->setData("by_code_{$code}_enabled_product", false);
            $this->parameters->setData("by_code_{$code}_enabled_cms", false);
            $this->parameters->setData("by_code_{$code}_enabled_home", false);
            $this->parameters->setData("by_code_{$code}_language_code", '');
            $this->parameters->setData("by_code_{$code}_use_canonical", false);
            $this->parameters->setData("by_code_{$code}_canonical_store", '');

            // By Id
            $this->parameters->setData("by_id_{$storeId}_enabled", false);
            $this->parameters->setData("by_id_{$storeId}_enabled_category", false);
            $this->parameters->setData("by_id_{$storeId}_enabled_product", false);
            $this->parameters->setData("by_id_{$storeId}_enabled_cms", false);
            $this->parameters->setData("by_id_{$storeId}_enabled_home", false);
            $this->parameters->setData("by_id_{$storeId}_language_code", '');
            $this->parameters->setData("by_id_{$storeId}_use_canonical", false);
            $this->parameters->setData("by_id_{$storeId}_canonical_store", '');


            if ($helperData) {
                $useCanonical = $helperData->isCanonicalUrlEnabled($storeId);
                $this->parameters->setData("by_code_{$code}_use_canonical", $useCanonical);
                $this->parameters->setData("by_id_{$storeId}_use_canonical", $useCanonical);

                $canonicalStore = $helperData->getCrossDomainStore($storeId);
                $this->parameters->setData("by_code_{$code}_canonical_store", $canonicalStore);
                $this->parameters->setData("by_id_{$storeId}_canonical_store", $canonicalStore);
            }
        }

        if (!$hrefLangHelper) {
            return $this->parameters;
        }

        // Calculate values

//        foreach ($hrefLangHelper->getAllEnabledStoreByType() as $store) {
//            $this->parameters->setData("by_id_{$store->getId()}_enabled", true);
//            $this->parameters->setData("by_code_{$store->getCode()}_enabled", true);
//        }

        foreach ($hrefLangHelper->getAllEnabledStoreByType('product') as $store) {
            $this->parameters->setData("by_id_{$store->getId()}_enabled_product", true);
            $this->parameters->setData("by_code_{$store->getCode()}_enabled_product", true);
        }

        foreach ($hrefLangHelper->getAllEnabledStoreByType('category') as $store) {
            $this->parameters->setData("by_id_{$store->getId()}_enabled_category", true);
            $this->parameters->setData("by_code_{$store->getCode()}_enabled_category", true);
        }

        foreach ($hrefLangHelper->getAllEnabledStoreByType('cms') as $store) {
            $this->parameters->setData("by_id_{$store->getId()}_enabled_cms", true);
            $this->parameters->setData("by_code_{$store->getCode()}_enabled_cms", true);
        }


        // TODO: Handle in the future
        $type = 'cms';
        $languageCodes = $hrefLangHelper->getHreflangLanguageCodes($type, 0);
        $countryCodes = $hrefLangHelper->getHreflangCountryCodes($type, 0);

        foreach ($languageCodes as $storeId => $languageCode) {
            $store = $this->storeManager->getStore($storeId);

            if (!empty($countryCodes[$storeId])) {
                $languageCode .= '-' . $countryCodes[$storeId];
            }

            $this->parameters->setData("by_id_{$store->getId()}_language_code", $languageCode);
            $this->parameters->setData("by_code_{$store->getCode()}_language_code", $languageCode);
        }

        return $this->parameters;
    }

    /**
     * @param int|string|StoreInterface|Store $store
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isEnabledForCategory($store = 0): bool
    {
        return $this->getValue('enabled_category', $store);
    }

    /**
     * @param int|string|StoreInterface|Store $store
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isEnabledForProduct($store = 0): bool
    {
        return $this->getValue('enabled_product', $store);
    }

    /**
     * @param int|string|StoreInterface|Store $store
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isEnabledForCms($store = 0): bool
    {
        return $this->getValue('enabled_cms', $store);
    }

    /**
     * @param $store
     * @return bool
     */
    public function isEnabledForHome($store = 0): bool
    {
        return $this->defaultConfiguration->isEnabledForHome($store);
    }

    /**
     * @param $store
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getLanguageCode($store = 0)
    {
        return $this->getValue('language_code', $store);
    }

    /**
     * @param $store
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function usesCanonical($store = 0): bool
    {
        return $this->getValue('use_canonical', $store);
    }

    /**
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCanonicalStore($store)
    {
        return $this->getValue('canonical_store', $store);
    }
}
